/*
 *      error_function.c
 *
 *      Copyright © 2010 Dionisio E Alonso <dalonso@gmail.com>
 *
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation, either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <math.h>
#include <stdio.h>
#include "/opt/mpich/gnu/include/mpi.h"
#include "simpson_mpi.h"
#include "../benchmark.h"

#define MASTER 0
#define INTERVALS 1000
#define MAX_RANGE 3.5

double f(double x);

int main(int argc, char **argv) {
	int i=0;
	double x=0, integ=0;
	int rank=0;

	MPI_Status status;
	MPI_Init(&argc, &argv);
	MPI_Comm_rank(MPI_COMM_WORLD, &rank);

	if (rank==MASTER) BENCH_START();
	for (x=-MAX_RANGE; x<=MAX_RANGE; x+=0.01) {
		integ = 2/sqrt(M_PI)*simpsons_rule(f, 0, x, INTERVALS, MPI_COMM_WORLD);
		if (rank==MASTER)
			printf("%.2f %.6f\n", x, integ);
	}
	if (rank==MASTER) BENCH_STOP();

	MPI_Finalize();
	return 0;
}

double f(double x) {
	return exp(-pow(x,2));
}
