/*
 *      simpson.c
 *
 *      Copyright © 2010 Dionisio E Alonso <dealonso@gmail.com>
 *
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation, either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


#include <omp.h>

double simpsons_rule(double (*f)(double), double a, double b, int intervals) {
	int i=0;
	double h=0, simpson=0;

	h = (b-a)/intervals;

	#pragma omp parallel for shared(h) private(i) reduction(+:simpson)
	for (i=0; i<=intervals; i++) {
		if (i==0 || i==intervals)
			simpson = simpson + f(a+i*h);
		else if (i%2 == 1)
			simpson = simpson + f(a+i*h)*4;
		else
			simpson = simpson + f(a+i*h)*2;
	}	/*
		 * It is used (a+i*h) because (a) is the start of the function
		 * interval and (i*h) is the amount computed so far.
		 */

	return h/3*simpson;
}
