/*
 *      error_function.c
 *
 *      Copyright © 2010 Dionisio E Alonso <dalonso@gmail.com>
 *
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation, either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <math.h>
#include <stdio.h>
#include "simpson_cuda.cuh"
#include "../benchmark.h"

#define INTERVALS 1000
#define MAX_RANGE 10
#define MAX_ITER 400
#define MAX_INF 20

double sum(double x, int n);

int main(void) {
	int i=0;
	double x=0, integ=0;
	int n=3;
	double sums=0;

BENCHMARK(
	for (x=0.01; x<=MAX_RANGE; x+=0.01) {
		printf("%.2f", x);

		for (i=3; i<=MAX_ITER; i++) {
			sums = sum(x, i);
			integ = simpsons_rule_c(x, MAX_INF, INTERVALS, i);

			printf(" %.6f %.6f %.6f", sums, integ, sums+integ);
		}

		printf("\n");
	}
)

	return 0;
}

double sum(double x, int n) {
	int i=0, num=-1;
	double sum=0, ret=0;

	sum = 1/(2*x)-1/(4*pow(x,3));

	for (i=3; i<n+1; i++) {
		num = -1*num*(2*i-3);
		sum = sum + num/(pow(2,i)*pow(x,(2*i-1)));
	}

	ret = 2/sqrt(M_PI)*exp(-pow(x,2))*sum;

	return ret;
}
