/*
 *      simpson_cuda.cu
 *
 *      Copyright © 2010 Dionisio E Alonso <dealonso@gmail.com>
 *
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation, either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


#include <cuda.h>
#include <stdlib.h>
#include <thrust/reduce.h>
#include <thrust/functional.h>
#include "cuda_utils.h"
#include "function.cu"

__global__ void simpson_kernel(float a, float h, int intervals, float *simpson, int n);

extern "C" float simpsons_rule_c(float a, float b, int intervals, int n) {
	float h=0, simpson=0, *ss=NULL, *d_res=NULL;

	h = (b-a)/intervals;

	ss = (float *) calloc(intervals+1, sizeof(float));

	CUDA_SAFE_CALL(cudaMalloc((void **) &d_res, (intervals+1)*sizeof(float)));
	CUDA_SAFE_CALL(cudaMemset((void *) d_res, 0, (intervals+1)*sizeof(float)));

	// Invoke kernel
	dim3 dim_block(BLOCK_SIZE);
	dim3 dim_grid(DIV_CEIL(intervals, dim_block.x));

	simpson_kernel<<<dim_grid, dim_block>>>(a, h, intervals, d_res, n);
	cudaThreadSynchronize();

	CUT_CHECK_ERROR("Kernel execution failed");
	// End kernel invocation

	CUDA_SAFE_CALL(cudaMemcpy(ss, d_res, (intervals+1)*sizeof(float), cudaMemcpyDeviceToHost));

	// Free device memory
	cudaFree(d_res);

	simpson = thrust::reduce(ss, ss+intervals+1);

	return h/3*simpson;
}

__global__ void simpson_kernel(float a, float h, int intervals, float *simpson, int n) {
	// Determine element to process from thread index
	const int i = blockIdx.x * blockDim.x + threadIdx.x;

	if (i<=intervals) {
		if (i==0 || i==intervals)
			simpson[i] = simpson[i] + f(a+i*h, n);
		else if (i%2 == 1)
			simpson[i] = simpson[i] + f(a+i*h, n)*4;
		else
			simpson[i] = simpson[i] + f(a+i*h, n)*2;
	}	/*
		 * It is used (a+i*h) because (a) is the start of the function
		 * interval and (i*h) is the amount computed so far.
		 */
}
