/*
 *      error_function.c
 *
 *      Copyright © 2010 Dionisio E Alonso <dalonso@gmail.com>
 *
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation, either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <math.h>
#include <stdio.h>
#include "/opt/mpich/gnu/include/mpi.h"
#include "simpson_mpi.h"
#include "../benchmark.h"

#define MASTER 0
#define INTERVALS 1000
#define MAX_RANGE 10
#define MAX_ITER 400
#define MAX_INF 20

double f(double x, int n);
double sum(double x, int n);

int main(int argc, char **argv) {
	int i=0;
	double x=0, integ=0;
	int n=3;
	double sums=0;
	int rank=0;

	MPI_Status status;
	MPI_Init(&argc, &argv);
	MPI_Comm_rank(MPI_COMM_WORLD, &rank);

	if (rank==MASTER) BENCH_START();
	for (x=0.01; x<=MAX_RANGE; x+=0.01) {
		if (rank==MASTER)
			printf("%.2f", x);

		for (i=3; i<=MAX_ITER; i++) {
			if (rank==MASTER)
				sums = sum(x, i);
			integ = simpsons_rule(f, x, MAX_INF, INTERVALS, MPI_COMM_WORLD, i);

			if (rank==MASTER)
				printf(" %.6f %.6f %.6f", sums, integ, sums+integ);
		}

		if (rank==MASTER)
			printf("\n");
	}
	if (rank==MASTER) BENCH_STOP();

	MPI_Finalize();
	return 0;
}

double f(double x, int n) {
	int i=0;
	double num=1;

	for (i=1; i<=n; i++) {
		num *= 2*i-1;
	}

	return powf(-1,n)*num/powf(2,n)*2/sqrtf(M_PI)*expf(-pow(x,2))/powf(x,2*n);
}

double sum(double x, int n) {
	int i=0, num=-1;
	double sum=0, ret=0;

	sum = 1/(2*x)-1/(4*pow(x,3));

	for (i=3; i<n+1; i++) {
		num = -1*num*(2*i-3);
		sum = sum + num/(pow(2,i)*pow(x,(2*i-1)));
	}

	ret = 2/sqrt(M_PI)*exp(-pow(x,2))*sum;

	return ret;
}
