/*
 *      simpson_mpi.c
 *
 *      Copyright © 2010 Dionisio E Alonso <dealonso@gmail.com>
 *
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation, either version 3 of the License, or
 *      (at your option) any later version.
 *
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *
 *      You should have received a copy of the GNU General Public License
 *      along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


#include "/opt/mpich/gnu/include/mpi.h"

#define MASTER 0

double simpsons_rule(double (*f)(double, int), double a, double b, int intervals, MPI_Comm comm, int n) {
	int i=0;
	double h=0, simpson=0, simpson_tot=0;
	int size=0, rank=0, interv_per_node=0, extra_interv_per_node=0, p=0;

	MPI_Comm_rank(comm, &rank);
	MPI_Comm_size(comm, &size);

	MPI_Bcast(&intervals, 1, MPI_INT, MASTER, comm);

	h = (b-a)/intervals;
	interv_per_node = intervals/size;
	extra_interv_per_node = intervals%size;

	for (i=0; i<interv_per_node; i++) {
		p = interv_per_node*rank+i;
		if (p==intervals-1) simpson+=f(a+p*h, n);
		if (p==0 || p==intervals)
			simpson = simpson + f(a+p*h, n);
		else if (i%2 == 1)
			simpson = simpson + f(a+p*h, n)*4;
		else
			simpson = simpson + f(a+p*h, n)*2;
	}	/*
		 * It is used (a+i*h) because (a) is the start of the function
		 * interval and (i*h) is the amount computed so far.
		 */

	MPI_Reduce(&simpson, &simpson_tot, 1, MPI_DOUBLE, MPI_SUM, MASTER, comm);

	return h/3*simpson_tot;
}
